<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1010Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS1010($params) {
        try {
            // Só envia itens se for CNPJ Matriz ou se for outro tipo de inscrição
            $idMatriz = isset($params['IDMATRIZ']) ? $params['IDMATRIZ'] : 'S';
            if ($idMatriz == 'S') {
                $sql = <<<SQL
SELECT EH.NREVENTOM as NRCHAVE,
       TM.NRTIPOMOVIMENT as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       EH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM FPA_EVENTOM EM
  JOIN FPA_EVENTOH EH ON EH.NRORG = EM.NRORG
                     AND EH.NREVENTOM = EM.NREVENTOM
                     AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH H1
                                               WHERE H1.NREVENTOM = EH.NREVENTOM
                                                 AND H1.NRORG = EH.NRORG
                                                 AND H1.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC))
  /*Tipos de Movimento sao usados como tabela*/
  JOIN FPA_TIPOMOVIMENT TM ON TM.NRTIPOMOVIMENT IN (1,2,4,10,19,22)
  /*Natureza da Rubrica*/
  JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, EC.NRTABELA, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-1010'
              WHERE EC.NRORG = :P_NRORG
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE, EC.NRTABELA
             UNION ALL
             SELECT EC.NRORG, EC.NRCHAVE, EC.NRTABELA, COUNT(*) AS ENVIOS
               FROM ESO_CADASTRO EC
              WHERE EC.NRORG = :P_NRORG
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                AND EC.NMEVENTO = 'S-1010'
              GROUP BY EC.NRORG, EC.NRCHAVE, EC.NRTABELA) ENV ON ENV.NRCHAVE = EH.NREVENTOM
                                                             AND ENV.NRTABELA = TM.NRTIPOMOVIMENT
 WHERE (EM.NRORG = :P_NRORG OR EM.NRORG = 0)
   AND EM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
   AND (EM.DTFIMVIGENCIA IS NULL OR EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
   AND (EM.NREVENTOM = :P_NREVENTOM OR :P_NREVENTOM IS NULL)
   AND (TM.NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT OR INSTR(','||:P_NRTIPOMOVIMENT||',', ','||TM.NRTIPOMOVIMENT||',') > 0 OR :P_NRTIPOMOVIMENT IS NULL)
 ORDER BY EH.NREVENTOM, TM.NRTIPOMOVIMENT, EH.DTMESCOMPETENC
SQL;
                $params = array(
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                    "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                    "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
                    "P_NREVENTOM" => isset($params['NREVENTOM']) ? $params['NREVENTOM'] : null,
                    "P_NRTIPOMOVIMENT" => isset($params['NRTIPOMOVIMENT']) ? $params['NRTIPOMOVIMENT'] : null,
                );
                
                $return = $this->connection->fetchAll($sql, $params);
                return $return;
            } else {
                throw new \Exception('HCM09000_ESO_EVENTO_MATRIZ_ERROR');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosS1010($params) {
        try {
            $dataHistorico = \HCM\Util\DateUtil::getDataDeString($params['DTMESCOMPETENC'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format("d/m/Y");
            
            $dados = $this->getDadosPrincipais($params['NRENVIO'], $params['NRCHAVE'], $params['NRTABELA'],  $params['NRESTRUTURAM'],  $params['NRTPAMBIENTE']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosPrincipais($nrenvio, $nrchave, $nrtabela, $nrestruturam, $nrtpambiente) {
        $sql = <<<SQL
/*getDadosS1010*/
SELECT EM.NREVENTOM CHAVE, TM.NRTIPOMOVIMENT NRTABELA,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --infoRubrica
       --ideRubrica
       EM.NREVENTOM codRubr,
       TM.NRTIPOMOVIMENT ideTabRubr,
       --idePeriodo
       TO_CHAR(GREATEST(EMP.DTINIVIGENCIA,EM.DTINIVIGENCIA,TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))),'YYYY-MM') iniValid,
       TO_CHAR(EM.DTFIMVIGENCIA,'YYYY-MM') fimValid,
       --dadosRubrica
       EH.NMEVENTOH dscRubr,
       NR.CDESOCIAL natRubr,
       DECODE(EH.IDEVENTODEMONST,'S','3',DECODE(EM.IDTIPOEVENTO,'PROVENTO','1','DESCONTO','2','ENCARGO','2','TOTAL','3')) tpRubr,
       NVL((SELECT MAX(INC.CDESOCIAL)
                FROM INCIDENCIACP INC 
                JOIN FPA_PROCINCIDEVENTO EVT ON INC.NRINCIDCP = EVT.NRINCIDCP 
                WHERE EVT.NRORG = :P_NRORG AND EVT.NREVENTOM = :P_NRCHAVE AND EVT.NRTIPOMOVIMENT = :P_NRTABELA), NVL(CP.CDESOCIAL,'00')) codIncCP,                
       NVL((SELECT MAX(IRF.CDESOCIAL)
                FROM INCIDENCIAIRRF IRF
                JOIN FPA_PROCINCIDEVENTO EVT ON IRF.NRINCIDIRRF = EVT.NRINCIDIRRF
                WHERE EVT.NRORG = :P_NRORG AND EVT.NREVENTOM = :P_NRCHAVE AND EVT.NRTIPOMOVIMENT = :P_NRTABELA), NVL(IRRF.CDESOCIAL,'09')) codIncIRRF,
       NVL(( SELECT MAX(IFGTS.CDESOCIAL)
                FROM INCIDENCIAFGTS IFGTS
                JOIN FPA_PROCINCIDEVENTO EVT ON IFGTS.NRINCIDFGTS = EVT.NRINCIDFGTS
                WHERE EVT.NRORG = :P_NRORG AND EVT.NREVENTOM = :P_NRCHAVE AND EVT.NRTIPOMOVIMENT = :P_NRTABELA), NVL(FGTS.CDESOCIAL,'00')) codIncFGTS,
       NVL(SIND.CDESOCIAL,'00') codIncSIND,       
       DECODE(EM.NREVENTOM, DSR.NREVENTOM, 'S', 'N') repDSR,
       DECODE(EM.NREVENTOM, SAL13.NREVENTOM, 'S', 'N') rep13,
       DECODE(EM.NREVENTOM, FERIAS.NREVENTOM, 'S', 'N') repFerias,
       DECODE(EM.NREVENTOM, AVISO.NREVENTOM, 'S', 'N') repAviso,
       SUBSTR(EH.DSOBSERVACAO,1,255) observacao,
       --ideProcessoCP : CRIAR SELECT
       --ideProcessoIRRF : CRIAR SELECT
       --ideProcessoFGTS : CRIAR SELECT
       --ideProcessoSIND : CRIAR SELECT
       '' novaValidade
       --TO_CHAR(EM.DTINIVIGENCIA,'YYYY-MM') novaValidade_iniValid,
       --TO_CHAR(EM.DTFIMVIGENCIA,'YYYY-MM') novaValidade_fimValid
  FROM FPA_EVENTOM EM
  JOIN FPA_EVENTOH EH ON EH.NREVENTOM = EM.NREVENTOM
                     AND EH.NRORG = EM.NRORG
                     AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH H1
                                               WHERE H1.NREVENTOM = EH.NREVENTOM
                                                 AND H1.NRORG = EH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Tipos de Movimento sao usados como tabela*/
  JOIN FPA_TIPOMOVIMENT TM ON TM.NRTIPOMOVIMENT IN (1,2,4,10,19,22)
  /*Natureza da Rubrica*/
  JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
  /*CONTRIBUICAO PREVIDENCIARIA*/
  LEFT JOIN (SELECT IE.NREVENTOM, TI.DSNRTIPOMOVIMENT, INC.CDESOCIAL
               FROM FPA_INCIDEVENTO IE
               JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
               JOIN INCIDENCIACP INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
              WHERE IE.NRTIPOINCIDE IN (2,5,8,11,17,20)
             ) CP ON CP.NREVENTOM = EM.NREVENTOM
                 AND INSTR(','||CP.DSNRTIPOMOVIMENT||',',','||TM.NRTIPOMOVIMENT||',') > 0
  /*IRRF*/
  LEFT JOIN (SELECT IE.NREVENTOM, TI.DSNRTIPOMOVIMENT, INC.CDESOCIAL
               FROM FPA_INCIDEVENTO IE
               JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
               JOIN INCIDENCIAIRRF INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
              WHERE IE.NRTIPOINCIDE IN (1,4,7,10,16,19)
             ) IRRF ON IRRF.NREVENTOM = EM.NREVENTOM
                 AND INSTR(','||IRRF.DSNRTIPOMOVIMENT||',',','||TM.NRTIPOMOVIMENT||',') > 0
  /*FGTS*/
  LEFT JOIN (SELECT IE.NREVENTOM, TI.DSNRTIPOMOVIMENT, INC.CDESOCIAL
               FROM FPA_INCIDEVENTO IE
               JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
               JOIN INCIDENCIAFGTS INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
              WHERE IE.NRTIPOINCIDE IN (3,6,9,12,18,21)
             ) FGTS ON FGTS.NREVENTOM = EM.NREVENTOM
                 AND INSTR(','||FGTS.DSNRTIPOMOVIMENT||',',','||TM.NRTIPOMOVIMENT||',') > 0
  /*CONTRIBUICAO SINDICAL*/
  LEFT JOIN (SELECT IE.NREVENTOM, TI.DSNRTIPOMOVIMENT, INC.CDESOCIAL
               FROM FPA_INCIDEVENTO IE
               JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
               JOIN INCIDENCIASIND INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
              WHERE IE.NRTIPOINCIDE IN (0)
             ) SIND ON SIND.NREVENTOM = EM.NREVENTOM
                 AND INSTR(','||SIND.DSNRTIPOMOVIMENT||',',','||TM.NRTIPOMOVIMENT||',') > 0
  /*REP DSR*/
  LEFT JOIN (SELECT DISTINCT IE.NREVENTOM
               FROM FPA_TIPOINCIDE TI
               JOIN FPA_INCIDEVENTO IE ON IE.NRTIPOINCIDE = TI.NRTIPOINCIDE
              WHERE TI.IDREPDSR = 'S') DSR ON DSR.NREVENTOM = EM.NREVENTOM
  /*REP 13*/
  LEFT JOIN (SELECT DISTINCT IE.NREVENTOM
               FROM FPA_TIPOINCIDE TI
               JOIN FPA_INCIDEVENTO IE ON IE.NRTIPOINCIDE = TI.NRTIPOINCIDE
              WHERE TI.IDREPDECTERC = 'S') SAL13 ON SAL13.NREVENTOM = EM.NREVENTOM
  /*REP FERIAS*/
  LEFT JOIN (SELECT DISTINCT IE.NREVENTOM
               FROM FPA_TIPOINCIDE TI
               JOIN FPA_INCIDEVENTO IE ON IE.NRTIPOINCIDE = TI.NRTIPOINCIDE
              WHERE TI.IDREPFERIAS = 'S') FERIAS ON FERIAS.NREVENTOM = EM.NREVENTOM
  /*REP AVISO*/
  LEFT JOIN (SELECT DISTINCT IE.NREVENTOM
               FROM FPA_TIPOINCIDE TI
               JOIN FPA_INCIDEVENTO IE ON IE.NRTIPOINCIDE = TI.NRTIPOINCIDE
              WHERE TI.IDREPAVISO = 'S') AVISO ON AVISO.NREVENTOM = EM.NREVENTOM
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE (EM.NRORG = :P_NRORG OR EM.NRORG = 0)
   AND EM.NREVENTOM = :P_NRCHAVE
   AND TM.NRTIPOMOVIMENT = :P_NRTABELA
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRTABELA" => $nrtabela,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidCadIniProdRest(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidCadIniProducao(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidCadIniNacioSimples(),
            
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS1010ideProcessoCP($params){
        $sql = <<<SQL
SELECT PRO.NRPROCESSADMJUD tpProc,
       PRO.CDPROCESSADMJUD nrProc,
       '1' extDecisao,
       IPF.NRITEMPROCADMJUD codSusp
       FROM FPA_PROCINCIDEVENTO EVT
       JOIN GPE_PROCESSADMJUD PRO ON PRO.NRORG = EVT.NRORG AND PRO.NRPROCESSADMJUD = EVT.NRPROCESSADMJUD
       LEFT JOIN (SELECT NRPROCESSADMJUD, MIN(NRITEMPROCADMJUD) AS NRITEMPROCADMJUD
               FROM GPE_ITEMPROCADMJUD
              WHERE NRORG = :P_NRORG
              GROUP BY NRPROCESSADMJUD) IPF ON PRO.NRPROCESSADMJUD = IPF.NRPROCESSADMJUD
       WHERE EVT.NRORG = :P_NRORG
       AND EVT.NREVENTOM = :P_NREVENTOM
       AND EVT.NRINCIDCP IS NOT NULL
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_NREVENTOM" => $params['CODRUBR']
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS1010ideProcessoIRRF($params){
        $sql = <<<SQL
SELECT PRO.CDPROCESSADMJUD nrProc,
       IPF.NRITEMPROCADMJUD codSusp
       FROM FPA_PROCINCIDEVENTO EVT
       JOIN GPE_PROCESSADMJUD PRO ON PRO.NRORG = EVT.NRORG AND PRO.NRPROCESSADMJUD = EVT.NRPROCESSADMJUD
       LEFT JOIN (SELECT NRPROCESSADMJUD, MIN(NRITEMPROCADMJUD) AS NRITEMPROCADMJUD
               FROM GPE_ITEMPROCADMJUD
              WHERE NRORG = :P_NRORG
              GROUP BY NRPROCESSADMJUD) IPF ON PRO.NRPROCESSADMJUD = IPF.NRPROCESSADMJUD
       WHERE EVT.NRORG = :P_NRORG
       AND EVT.NREVENTOM = :P_NREVENTOM
       AND EVT.NRINCIDIRRF IS NOT NULL
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_NREVENTOM" => $params['CODRUBR']
        );
        return $this->connection->fetchAll($sql, $params);
    }

    public function getDadosS1010ideProcessoFGTS($params){
        $sql = <<<SQL
SELECT PRO.CDPROCESSADMJUD nrProc
       FROM FPA_PROCINCIDEVENTO EVT
       JOIN GPE_PROCESSADMJUD PRO ON PRO.NRORG = EVT.NRORG AND PRO.NRPROCESSADMJUD = EVT.NRPROCESSADMJUD
       LEFT JOIN (SELECT NRPROCESSADMJUD, MIN(NRITEMPROCADMJUD) AS NRITEMPROCADMJUD
               FROM GPE_ITEMPROCADMJUD
              WHERE NRORG = :P_NRORG
              GROUP BY NRPROCESSADMJUD) IPF ON PRO.NRPROCESSADMJUD = IPF.NRPROCESSADMJUD
       WHERE EVT.NRORG = :P_NRORG
       AND EVT.NREVENTOM = :P_NREVENTOM
       AND EVT.NRINCIDFGTS IS NOT NULL
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_NREVENTOM" => $params['CODRUBR']
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
}