<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class NormaClausulaController extends Crud {
    
    protected $normaClausulaService;
    protected $dataSourceName = 'norma';
    protected $dataSourceName1 = 'clausula';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\SST\Services\NormaClausulaService $normaClausulaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->normaClausulaService = $normaClausulaService;
        parent::__construct($dataSourceManager);
    }

    public function getNormaClausula(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setNormaClausula(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('CDNORMA', $row)) {
                    if ($row['CDNORMA'] === '') {
                        $row['CDNORMA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'NORMA', 1, 12, true);
                    }
                } else {
                    $row['CDNORMA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'NORMA', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteNormaClausula(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->normaClausulaService->deleteNormaClausula($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function getClausula(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setClausula(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "clausula";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('CDCLAUSULA', $row)) {
                    if ($row['CDCLAUSULA'] === '') {
                        $row['CDCLAUSULA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'CLAUSULA', 1, 12, true);
                    }
                } else {
                    $row['CDCLAUSULA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'CLAUSULA', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteClausula(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "clausula";
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 