<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\SST\Services\TipoAcidenteService;

class TipoAcidenteController extends Crud {

    protected $dataSourceName = 'sst_tipoacidente';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, TipoAcidenteService $tipoAcidenteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->tipoAcidenteService = $tipoAcidenteService;
        parent::__construct($dataSourceManager);
    }

    public function getTipoacidente(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setTipoacidente(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $existPrimaryKey = false;
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existPrimaryKey = $this->tipoAcidenteService->validaRestricaoChavePrimaria(
                    $row['CDESOCIAL'], 
                    $this->hcmParameters->getNrorgTrabalho(), 
                    $this->hcmParameters->getNrorgPadrao(),
                    $row['NRTIPOACIDENTE'] 
                );
                
                if(!$existPrimaryKey){
                    if (array_key_exists('NRTIPOACIDENTE', $row)) {
                        if ($row['NRTIPOACIDENTE'] === '') {
                            $row['NRTIPOACIDENTE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_TIPOACIDENTE', 1, 12, true);
                        }
                    } else {
                        $row['NRTIPOACIDENTE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_TIPOACIDENTE', 1, 12, true);
                    }
                }
            }
        
            if($existPrimaryKey)
                $response->addNotification(new DTO\Response\Notification("DUPLICATE_PRIMARY_KEY_ERROR", DTO\Response\Notification::TYPE_ERROR));    
            else
                parent::save($request, $response);
        
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteTipoacidente(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 